#include <windows.h>

const char decimalSeparator = ',';

HHOOK hhookSysMsg = NULL; 

/** KeyboardProc : Hook callback
 *
 * This callback :
 *  - tests if the key should be processed
 *  - tests if the key is the decimal separator key
 *  - send the new key to the application
 *
 * Notice : a call to the next hook (CallNextHookEx) would be better, but
 *    that suppose to have the value resulting of SetWindowsHook, which is 
 *    not very handy if several threads have to be hooked...
 */
LRESULT CALLBACK KeyboardProc(int nCode, WPARAM wParam, LPARAM lParam) 
{ 
 	// Test if we process the message
    if (! (nCode < 0)) 
	{
		// Test if it is VK_DECIMAL and KeyDown
		if ( (wParam == VK_DECIMAL)  && ((lParam &  0x80000000) == 0) )
		{

			HWND theWnd;
			theWnd = GetFocus();

			// [ If we decide to replace VK_DECIMAL only in Calc, some code may be needed here]
			
			PostMessage(theWnd, WM_CHAR, decimalSeparator, lParam);
			
			return true;
		}
	}
	// Call next hook (this is optional, but proper)
    return false; //CallNextHookEx(hhookSysMsg, nCode, wParam, lParam); 
} 

/** registerHook : This function registers the Hook
 * 
 * This is the local version of the hook. This should be less intrusive, and work with 
 * all other software, but as a drawback you must call this function in each thread you want
 * the decimal separator to be replaced.
 *
 * @return a pointer to the hook. Keep this to unregister it.
 */
HHOOK registerHook()
{
	return SetWindowsHookEx(WH_KEYBOARD,  (HOOKPROC)KeyboardProc, NULL, GetCurrentThreadId()); 
}

/** unregisterHook
 *
 * As it is a local hook, unregister hook is not really usefull (as it will be unloaded when the 
 * thread terminate), but it is proper...
 * 
 * @param hhookSysMsg is the pointer returned by registerHook.
 */
void unregisterHook(HHOOK hhookSysMsg)
{
	if (hhookSysMsg != NULL)
	{
		UnhookWindowsHookEx(hhookSysMsg);
	}
}
